/*
 * ca-lava-data.c
 * Convenience routines to push data around between the tasks
 */

#include <pvm3.h>

#include "v2int.h"
#include "v2float.h"
#include "exn.h"
#include "utils.h"
#include "ca-lava-data.h"

void send_config(int tid, const worker_info *info, v2int cell_dim, int step_count, int step_interval, float timestep, float cell_length, float solidus, const float coverage[], const float viscosity[], float cooling) {
  if ((pvm_packf("%+ %2d%4d %2d %d %d %f %f %f %2f %2f %f", PvmDataDefault,
		 info->org.a, info->next, cell_dim.a,
		 step_count, step_interval,
		 timestep, cell_length, solidus, coverage, viscosity, cooling) < 0)
      || (pvm_send(tid, MSG_CONFIG) < 0))
    EXN_THROW(EXN_CODE_PVM, "couldn't send configuration");
}

void recv_config(int tid, v2int *org, int next[], v2int *cell_dim, int *step_count, int *step_interval, float *timestep, float *cell_length, float *solidus, float coverage[], float viscosity[], float *cooling) {
  if ((pvm_recv(tid, MSG_CONFIG) < 0)
      || (pvm_unpackf("%2d%4d %2d %d %d %f %f %f %2f %2f %f",
		      org->a, next, cell_dim->a,
		      step_count, step_interval,
		      timestep, cell_length, solidus, coverage, viscosity, cooling) < 0))
    EXN_THROW(EXN_CODE_PVM, "couldn't receive configuration");
}

void send_board(int tid, const cell board[], v2int org, v2int dim, v2int pos, v2int size) {
  int i;

  V2INT_ADD(org, pos);
  if (pvm_packf("%+ %2d %2d", PvmDataDefault, org.a, size.a) < 0)
    EXN_THROW(EXN_CODE_PVM, "couldn't pack board data header");
  
  for (i = 0; i < size.y; i++, pos.y++) {
    if (pvm_pkfloat((float *)&board[V2INT_OFS(pos, dim)],
		    sizeof(cell) / sizeof(float) * size.x, 1) < 0)
      EXN_THROW(EXN_CODE_PVM, "couldn't pack board data");
  }
  
  if (pvm_send(tid, MSG_DATA) < 0)
    EXN_THROW(EXN_CODE_PVM, "couldn't send board data");
}

void recv_board(int tid, cell board[], v2int org, v2int dim) {
  v2int pos, size;
  int i;

  if ((pvm_recv(tid, MSG_DATA) < 0)
      || (pvm_unpackf("%2d %2d", pos.a, size.a) < 0))
    EXN_THROW(EXN_CODE_PVM, "couldn't receive board data");

  V2INT_SUB(pos, org);

  for (i = 0; i < size.y; i++, pos.y++) {
    if (pvm_upkfloat((float *)&board[V2INT_OFS(pos, dim)],
		     sizeof(cell) / sizeof(float) * size.x, 1) < 0)
      EXN_THROW(EXN_CODE_PVM, "couldn't unpack board data");
  }
}

/* vim:set shiftwidth=2 softtabstop=2: */
