/*!
 * @header Coordinate Lookup
 * Small helper to do coordinate lookup in the peek utilities.
 */

#ifndef __COORD_LOOKUP__
#define __COORD_LOOKUP__

/*!
 * @function coord_lookup
 * Find the index into a coordinate array that corresponds to a given
 * coordinate value. Coordinates are considered matching if
 * ref >= key <= ref + d, where ref is the element of the coordinate
 * array being tested and d the distance between the first two coordinates
 * in the coordinate array.
 * @param grid NetCDF handle for the simulation grid
 * @param coord name of the coordinate dimension/variable
 * @param key coordinate to look up in the form "<float>d" or index that
 * doesn't need to be looked up in the form "<int>"
 * @param dim place to store the coordinate dimension length.
 * May not be NULL.
 * @return the index of the first matching element in the coordinate
 * array. An exception is thrown if no matching entry is found
 */
size_t coord_lookup(int grid, const char *coord, size_t *dim, const char *key);

#endif /* __COORD_LOOKUP__ */

/* vim:set shiftwidth=2 softtabstop=2: */
